@@ -8,19 +8,15 @@
 """
 **Enlighten counter submodule**
 
-Provides Counter base class
+Provides Counter and SubConter classes
 """
 
 import platform
 import sys
 import time
 
-try:
-    from collections.abc import Iterable
-except ImportError:  # pragma: no cover(Python 2)
-    from collections import Iterable
-
-from blessed.colorspace import X11_COLORNAMES_TO_RGB
+from enlighten._basecounter import BaseCounter, PrintableCounter
+from enlighten._util import format_time
 
 COUNTER_FMT = u'{desc}{desc_pad}{count:d} {unit}{unit_pad}' + \
               u'[{elapsed}, {rate:.2f}{unit_pad}{unit}/s]{fill}'
@@ -28,6 +24,8 @@ COUNTER_FMT = u'{desc}{desc_pad}{count:d} {unit}{unit_pad}' + \
 BAR_FMT = u'{desc}{desc_pad}{percentage:3.0f}%|{bar}| {count:{len_total}d}/{total:d} ' + \
           u'[{elapsed}<{eta}, {rate:.2f}{unit_pad}{unit}/s]'
 
+STATUS_FMT = u'{message}'
+
 # Even with cp65001, Windows doesn't seem to support all unicode characters
 if platform.system() == 'Windows':  # pragma: no cover(Windows)
     SERIES_STD = u' ▌█'
@@ -42,141 +40,6 @@ except UnicodeEncodeError:  # pragma: no cover(Non-unicode Terminal)
 except (AttributeError, TypeError):  # pragma: no cover(Non-standard Terminal)
     pass
 
-COLORS_16 = ('black', 'red', 'green', 'yellow', 'blue', 'magenta', 'cyan', 'white',
-             'bright_black', 'bright_red', 'bright_green', 'bright_yellow',
-             'bright_blue', 'bright_magenta', 'bright_cyan', 'bright_white')
-
-try:
-    BASESTRING = basestring
-except NameError:
-    BASESTRING = str
-
-
-def _format_time(seconds):
-    """
-    Args:
-        seconds (float): amount of time
-
-    Format time string for eta and elapsed
-    """
-
-    # Always do minutes and seconds in mm:ss format
-    minutes = seconds // 60
-    hours = minutes // 60
-    rtn = u'{0:02.0f}:{1:02.0f}'.format(minutes % 60, seconds % 60)
-
-    #  Add hours if there are any
-    if hours:
-
-        rtn = u'{0:d}h {1}'.format(int(hours % 24), rtn)
-
-        #  Add days if there are any
-        days = int(hours // 24)
-        if days:
-            rtn = u'{0:d}d {1}'.format(days, rtn)
-
-    return rtn
-
-
-class BaseCounter(object):
-    """
-    Args:
-        manager(:py:class:`Manager`): Manager instance. Required.
-        color(str): Color as a string or RGB tuple (Default: None)
-
-    Base class for counters
-    """
-
-    __slots__ = ('_color', 'count', 'manager', 'start_count')
-    _repr_attrs = ('count', 'color')
-
-    def __repr__(self):
-
-        params = []
-        for attr in self._repr_attrs:
-            value = getattr(self, attr)
-            if value is not None:
-                params.append('%s=%r' % (attr, value))
-
-        return '%s(%s)' % (self.__class__.__name__, ', '.join(params))
-
-    def __init__(self, **kwargs):
-
-        self.count = self.start_count = kwargs.get('count', 0)
-        self._color = None
-
-        self.manager = kwargs.get('manager', None)
-        if self.manager is None:
-            raise TypeError('manager must be specified')
-
-        self.color = kwargs.get('color', None)
-
-    @property
-    def color(self):
-        """
-        Color property
-        Preferred to be a string or iterable of three integers for RGB
-        Single integer supported for backwards compatibility
-        """
-
-        color = self._color
-        return color if color is None else color[0]
-
-    @color.setter
-    def color(self, value):
-
-        if value is None:
-            self._color = None
-        elif isinstance(value, int) and 0 <= value <= 255:
-            self._color = (value, self.manager.term.color(value))
-        elif isinstance(value, BASESTRING):
-            if value not in COLORS_16 or value not in X11_COLORNAMES_TO_RGB:
-                raise AttributeError('Invalid color specified: %s' % value)
-            self._color = (value, getattr(self.manager.term, value))
-        elif isinstance(value, Iterable) and \
-                len(value) == 3 and \
-                all(isinstance(_, int) and 0 <= _ <= 255 for _ in value):
-            self._color = (value, self.manager.term.color_rgb(*value))
-        else:
-            raise AttributeError('Invalid color specified: %s' % repr(value))
-
-    def _colorize(self, content):
-        """
-        Args:
-            content(str): Color as a string or number 0 - 255 (Default: None)
-
-        Returns:
-            :py:class:`str`: content formatted with color
-
-        Format ``content`` with the color specified for this progress bar
-
-        If no color is specified for this instance, the content is returned unmodified
-        """
-
-        # No color specified
-        if self._color is None:
-            return content
-
-        # Used spec cached by color.setter
-        return self._color[1](content)
-
-    def update(self, incr=1, force=False):
-        """
-        Placeholder for update method
-        """
-
-        raise NotImplementedError
-
-    def __call__(self, *args):
-
-        for iterable in args:
-            if not isinstance(iterable, Iterable):
-                raise TypeError('Argument type %s is not iterable' % type(iterable).__name__)
-
-            for element in iterable:
-                yield element
-                self.update()
-
 
 class SubCounter(BaseCounter):
     """
@@ -216,7 +79,7 @@ class SubCounter(BaseCounter):
         self.parent = parent
         self.all_fields = all_fields
 
-    def update(self, incr=1, force=False):
+    def update(self, incr=1, force=False):  # pylint: disable=arguments-differ
         """
         Args:
             incr(int): Amount to increment ``count`` (Default: 1)
@@ -267,20 +130,23 @@ class SubCounter(BaseCounter):
             raise ValueError('source must be parent or peer')
 
 
-class Counter(BaseCounter):
+class Counter(PrintableCounter):
     """
     .. spelling::
         desc
         len
+        seagreen
+        peru
 
     Args:
-        additional_fields(dict): Additional fields used for :ref:`formating <counter_format>`
         bar_format(str): Progress bar format, see :ref:`Format <counter_format>` below
         count(int): Initial count (Default: 0)
         counter_format(str): Counter format, see :ref:`Format <counter_format>` below
         color(str): Series color as a string or RGB tuple see :ref:`Series Color <series_color>`
         desc(str): Description
         enabled(bool): Status (Default: :py:data:`True`)
+        fill(str): Fill character used for ``counter_format`` (Default: ' ')
+        fields(dict): Additional fields used for :ref:`formatting <counter_format>`
         leave(True): Leave progress bar after closing (Default: :py:data:`True`)
         manager(:py:class:`Manager`): Manager instance. Creates instance if not specified.
         min_delta(float): Minimum time, in seconds, between refreshes (Default: 0.1)
@@ -350,6 +216,9 @@ class Counter(BaseCounter):
         For backward compatibility, a color can be expressed as an integer 0 - 255, but this
         is deprecated in favor of named or RGB colors.
 
+        Compound colors, such as 'white_on_seagreen', 'bold_red', or 'underline_on_peru' are
+        also supported.
+
         If a terminal is not capable of 24-bit color, and is given a color outside of its
         range, the color will be downconverted to a supported color.
 
@@ -427,9 +296,10 @@ class Counter(BaseCounter):
         - percentage(:py:class:`float`) - Percentage complete
         - total(:py:class:`int`) - Value of ``total``
 
-        Addition fields for ``counter_format`` only:
+        Additional fields for ``counter_format`` only:
 
-        - fill(:py:class:`str`) - blank spaces, number needed to fill line
+        - fill(:py:class:`str`) - Filled with :py:attr:`fill` until line is width of terminal.
+          May be used multiple times. Minimum width is 3.
 
         Additional fields when subcounters are used:
 
@@ -453,17 +323,22 @@ class Counter(BaseCounter):
 
         User-defined fields:
 
-            The ``additional_fields`` parameter can be used to pass a dictionary of additional
+            Users can define fields in two ways, the ``fields`` parameter and by passing keyword
+            arguments to :py:meth:`Manager.counter` or :py:meth:`Counter.update`
+
+            The ``fields`` parameter can be used to pass a dictionary of additional
             user-defined fields. The dictionary values can be updated after initialization to allow
             for dynamic fields. Any fields that share names with built-in fields are ignored.
 
+            If fields are passed as keyword arguments to :py:meth:`Manager.counter` or
+            :py:meth:`Counter.update`, they take precedent over the ``fields`` parameter.
 
     .. _counter_offset:
 
     **Offset**
 
         When ``offset`` is :py:data:`None`, the width of the bar portion of the progress bar and
-        the fill characters for counter will be automatically determined,
+        the fill size for counter will be automatically determined,
         taking into account terminal escape sequences that may be included in the string.
 
         Under special circumstances, and to permit backward compatibility, ``offset`` may be
@@ -509,9 +384,8 @@ class Counter(BaseCounter):
     """
     # pylint: disable=too-many-instance-attributes
 
-    __slots__ = ('additional_fields', 'bar_format', 'counter_format', 'desc', 'enabled',
-                 'last_update', 'leave', 'manager', 'min_delta', 'offset', 'series', 'start',
-                 'total', 'unit', '_subcounters')
+    __slots__ = ('bar_format', 'counter_format', 'desc', 'fields', 'manager',
+                 'offset', 'series', 'total', 'unit', '_fields', '_subcounters')
     _repr_attrs = ('desc', 'total', 'count', 'unit', 'color')
 
     # pylint: disable=too-many-arguments
@@ -519,35 +393,18 @@ class Counter(BaseCounter):
 
         super(Counter, self).__init__(**kwargs)
 
-        self.additional_fields = kwargs.get('additional_fields', {})
-        self.bar_format = kwargs.get('bar_format', BAR_FMT)
-        self.counter_format = kwargs.get('counter_format', COUNTER_FMT)
-        self.desc = kwargs.get('desc', None)
-        self.enabled = kwargs.get('enabled', True)
-        self.leave = kwargs.get('leave', True)
-        self.min_delta = kwargs.get('min_delta', 0.1)
-        self.offset = kwargs.get('offset', None)
-        self.series = kwargs.get('series', SERIES_STD)
-        self.total = kwargs.get('total', None)
-        self.unit = kwargs.get('unit', None)
+        # Accept additional_fields for backwards compatibility
+        self.fields = kwargs.pop('fields', kwargs.pop('additional_fields', {}))
+        self.bar_format = kwargs.pop('bar_format', BAR_FMT)
+        self.counter_format = kwargs.pop('counter_format', COUNTER_FMT)
+        self.desc = kwargs.pop('desc', None)
+        self.offset = kwargs.pop('offset', None)
+        self.series = kwargs.pop('series', SERIES_STD)
+        self.total = kwargs.pop('total', None)
+        self.unit = kwargs.pop('unit', None)
+        self._fields = kwargs
         self._subcounters = []
 
-        self.last_update = self.start = time.time()
-
-    def __enter__(self):
-        return self
-
-    def __exit__(self, *args):
-        self.close()
-
-    @property
-    def position(self):
-        """
-        Fetch position from the manager
-        """
-
-        return self.manager.counters.get(self, 0)
-
     @property
     def elapsed(self):
         """
@@ -570,31 +427,6 @@ class Counter(BaseCounter):
 
         return sum(subcounter.count for subcounter in self._subcounters)
 
-    def clear(self, flush=True):
-        """
-        Args:
-            flush(bool): Flush stream after clearing progress bar (Default:True)
-
-        Clear progress bar
-        """
-
-        if self.enabled:
-            self.manager.write(flush=flush, position=self.position)
-
-    def close(self, clear=False):
-        """
-        Do final refresh and remove from manager
-
-        If ``leave`` is True, the default, the effect is the same as :py:meth:`refresh`.
-        """
-
-        if clear and not self.leave:
-            self.clear()
-        else:
-            self.refresh()
-
-        self.manager.remove(self)
-
     def _get_subcounters(self, elapsed, bar_fields=True):
         """
         Args:
@@ -645,7 +477,7 @@ class Counter(BaseCounter):
                 if self.total == 0:
                     fields['eta_{0}'.format(num)] = u'00:00'
                 elif rate:
-                    fields['eta_{0}'.format(num)] = _format_time((self.total - interations) / rate)
+                    fields['eta_{0}'.format(num)] = format_time((self.total - interations) / rate)
                 else:
                     fields['eta_{0}'.format(num)] = u'?'
 
@@ -668,7 +500,8 @@ class Counter(BaseCounter):
 
         iterations = abs(self.count - self.start_count)
 
-        fields = self.additional_fields.copy()
+        fields = self.fields.copy()
+        fields.update(self._fields)
         fields.update({'bar': u'{0}',
                        'count': self.count,
                        'desc': self.desc or u'',
@@ -681,7 +514,7 @@ class Counter(BaseCounter):
         if elapsed is None:
             elapsed = self.elapsed
 
-        fields['elapsed'] = _format_time(elapsed)
+        fields['elapsed'] = format_time(elapsed)
 
         # Get rate. Elapsed could be 0 if counter was not updated and has a zero total.
         if elapsed:
@@ -707,7 +540,7 @@ class Counter(BaseCounter):
                 # Get eta
                 if fields['rate']:
                     # Use iterations so a counter running backwards is accurate
-                    fields['eta'] = _format_time((self.total - iterations) / fields['rate'])
+                    fields['eta'] = format_time((self.total - iterations) / fields['rate'])
                 else:
                     fields['eta'] = u'?'
 
@@ -726,8 +559,7 @@ class Counter(BaseCounter):
             try:
                 rtn = self.bar_format.format(**fields)
             except KeyError as e:
-                raise ValueError('%r specified in format, but not present in additional_fields' %
-                                 e.args[0])
+                raise ValueError('%r specified in format, but not provided' % e.args[0])
 
             # Format the bar
             if self.offset is None:
@@ -767,35 +599,16 @@ class Counter(BaseCounter):
         try:
             rtn = self.counter_format.format(**fields)
         except KeyError as e:
-            raise ValueError('%r specified in format, but not present in additional_fields' %
-                             e.args[0])
+            raise ValueError('%r specified in format, but not provided' % e.args[0])
 
-        if self.offset is None:
-            ret = rtn.format(u' ' * (width - self.manager.term.length(rtn) + 3))
-        else:
-            # Offset was explicitly given
-            ret = rtn.format(u' ' * (width - len(rtn) + self.offset + 3))
+        return self._fill_text(rtn, width, offset=self.offset)
 
-        return ret
-
-    def refresh(self, flush=True, elapsed=None):
-        """
-        Args:
-            flush(bool): Flush stream after writing progress bar (Default:True)
-            elapsed(float): Time since started. Automatically determined if :py:data:`None`
-
-        Redraw progress bar
-        """
-
-        if self.enabled:
-            self.manager.write(output=self.format(elapsed=elapsed),
-                               flush=flush, position=self.position)
-
-    def update(self, incr=1, force=False):
+    def update(self, incr=1, force=False, **fields):  # pylint: disable=arguments-differ
         """
         Args:
             incr(int): Amount to increment ``count`` (Default: 1)
             force(bool): Force refresh even if ``min_delta`` has not been reached
+            fields(dict): Fields for for :ref:`formatting <counter_format>`
 
         Increment progress bar and redraw
 
@@ -803,12 +616,12 @@ class Counter(BaseCounter):
         """
 
         self.count += incr
+        self._fields.update(fields)
         if self.enabled:
             currentTime = time.time()
             # Update if force, 100%, or minimum delta has been reached
             if force or self.count == self.total or \
                     currentTime - self.last_update >= self.min_delta:
-                self.last_update = currentTime
                 self.refresh(elapsed=currentTime - self.start)
 
     def add_subcounter(self, color, count=0, all_fields=False):
