@@ -26,7 +26,7 @@ def initialize(manager, initials=15):
     # Simulated preparation
     pbar = manager.counter(total=initials, desc='Initializing:', unit='initials')
     for num in range(initials):  # pylint: disable=unused-variable
-        time.sleep(random.uniform(0.1, 0.5))  # Random processing time
+        time.sleep(random.uniform(0.05, 0.25))  # Random processing time
         pbar.update()
     pbar.close()
 
@@ -37,9 +37,21 @@ def main():
     """
 
     with enlighten.get_manager() as manager:
+        status = manager.status_bar(status_format=u'Enlighten{fill}Stage: {demo}{fill}{elapsed}',
+                                    color='bold_underline_bright_white_on_lightslategray',
+                                    justify=enlighten.Justify.CENTER, demo='Initializing',
+                                    autorefresh=True, min_delta=0.5)
+        docs = manager.term.link('https://python-enlighten.readthedocs.io/en/stable/examples.html',
+                                 'Read the Docs')
+        manager.status_bar(' More examples on %s! ' % docs, position=1, fill='-',
+                           justify=enlighten.Justify.CENTER)
+
         initialize(manager, 15)
-        load(manager, 80)
-        run_tests(manager, 40)
+        status.update(demo='Loading')
+        load(manager, 40)
+        status.update(demo='Testing')
+        run_tests(manager, 20)
+        status.update(demo='File Processing')
         process_files(manager)
 
 
