@@ -10,13 +10,12 @@ Test module for enlighten._counter and enlighten.counter
 """
 
 import time
-from types import GeneratorType
 
 from enlighten import Counter, Manager
 import enlighten._counter
 from enlighten._manager import NEEDS_UNICODE_HELP
 
-from tests import TestCase, mock, MockManager, MockTTY, MockCounter, MockBaseCounter
+from tests import TestCase, mock, MockManager, MockTTY, MockCounter
 
 
 # pylint: disable=missing-docstring, protected-access, too-many-public-methods
@@ -24,141 +23,6 @@ SERIES_STD = u' ▏▎▍▌▋▊▉█'
 BLOCK = enlighten._counter.SERIES_STD[-1]
 
 
-class TestFormatTime(TestCase):
-    """
-    Test cases for :py:func:`_format_time`
-    """
-
-    def test_seconds(self):
-
-        self.assertEqual(enlighten._counter._format_time(0), '00:00')
-        self.assertEqual(enlighten._counter._format_time(6), '00:06')
-        self.assertEqual(enlighten._counter._format_time(42), '00:42')
-
-    def test_minutes(self):
-
-        self.assertEqual(enlighten._counter._format_time(60), '01:00')
-        self.assertEqual(enlighten._counter._format_time(128), '02:08')
-        self.assertEqual(enlighten._counter._format_time(1684), '28:04')
-
-    def test_hours(self):
-
-        self.assertEqual(enlighten._counter._format_time(3600), '1h 00:00')
-        self.assertEqual(enlighten._counter._format_time(43980), '12h 13:00')
-        self.assertEqual(enlighten._counter._format_time(43998), '12h 13:18')
-
-    def test_days(self):
-
-        self.assertEqual(enlighten._counter._format_time(86400), '1d 0h 00:00')
-        self.assertEqual(enlighten._counter._format_time(1447597), '16d 18h 06:37')
-
-
-class TestBaseCounter(TestCase):
-    """
-    Test the BaseCounter class
-    """
-
-    def setUp(self):
-        self.tty = MockTTY()
-        self.manager = MockManager(stream=self.tty.stdout)
-
-    def tearDown(self):
-        self.tty.close()
-
-    def test_init_default(self):
-        """Ensure default values are set"""
-        counter = enlighten._counter.BaseCounter(manager=self.manager)
-        self.assertIsNone(counter.color)
-        self.assertIsNone(counter.color)
-        self.assertIs(counter.manager, self.manager)
-        self.assertEqual(counter.count, 0)
-        self.assertEqual(counter.start_count, 0)
-
-    def test_no_manager(self):
-        """Raise an error if there is no manager specified"""
-        with self.assertRaisesRegex(TypeError, 'manager must be specified'):
-            enlighten._counter.BaseCounter()
-
-    def test_color_invalid(self):
-        """Color must be a valid string, RGB, or int 0 - 255"""
-        # Unsupported type
-        with self.assertRaisesRegex(AttributeError, 'Invalid color specified: 1.0'):
-            enlighten._counter.BaseCounter(manager=self.manager, color=1.0)
-
-        # Invalid String
-        with self.assertRaisesRegex(AttributeError, 'Invalid color specified: buggersnot'):
-            enlighten._counter.BaseCounter(manager=self.manager, color='buggersnot')
-
-        # Invalid integer
-        with self.assertRaisesRegex(AttributeError, 'Invalid color specified: -1'):
-            enlighten._counter.BaseCounter(manager=self.manager, color=-1)
-        with self.assertRaisesRegex(AttributeError, 'Invalid color specified: 256'):
-            enlighten._counter.BaseCounter(manager=self.manager, color=256)
-
-        # Invalid iterable
-        with self.assertRaisesRegex(AttributeError, r'Invalid color specified: \[\]'):
-            enlighten._counter.BaseCounter(manager=self.manager, color=[])
-        with self.assertRaisesRegex(AttributeError, r'Invalid color specified: \[1\]'):
-            enlighten._counter.BaseCounter(manager=self.manager, color=[1])
-        with self.assertRaisesRegex(AttributeError, r'Invalid color specified: \(1, 2\)'):
-            enlighten._counter.BaseCounter(manager=self.manager, color=(1, 2))
-        with self.assertRaisesRegex(AttributeError, r'Invalid color specified: \(1, 2, 3, 4\)'):
-            enlighten._counter.BaseCounter(manager=self.manager, color=(1, 2, 3, 4))
-
-    def test_colorize_none(self):
-        """If color is None, return content unchanged"""
-        counter = enlighten._counter.BaseCounter(manager=self.manager)
-        self.assertEqual(counter._colorize('test'), 'test')
-
-    def test_colorize_string(self):
-        """Return string formatted with color (string)"""
-        counter = enlighten._counter.BaseCounter(manager=self.manager, color='red')
-        self.assertEqual(counter.color, 'red')
-        self.assertEqual(counter._color, ('red', self.manager.term.red))
-        self.assertNotEqual(counter._colorize('test'), 'test')
-        self.assertEqual(counter._colorize('test'), self.manager.term.red('test'))
-
-    def test_colorize_int(self):
-        """Return string formatted with color (int)"""
-        counter = enlighten._counter.BaseCounter(manager=self.manager, color=40)
-        self.assertEqual(counter.color, 40)
-        self.assertEqual(counter._color, (40, self.manager.term.color(40)))
-        self.assertNotEqual(counter._colorize('test'), 'test')
-        self.assertEqual(counter._colorize('test'), self.manager.term.color(40)('test'))
-
-    def test_colorize_rgb(self):
-        """Return string formatted with color (RGB)"""
-        counter = enlighten._counter.BaseCounter(manager=self.manager, color=(50, 40, 60))
-        self.assertEqual(counter.color, (50, 40, 60))
-        self.assertEqual(counter._color, ((50, 40, 60), self.manager.term.color_rgb(50, 40, 60)))
-        self.assertNotEqual(counter._colorize('test'), 'test')
-        self.assertEqual(counter._colorize('test'), self.manager.term.color_rgb(50, 40, 60)('test'))
-
-    def test_call(self):
-        """Returns generator when used as a function"""
-
-        # Bad arguments
-        counter = MockBaseCounter(manager=self.manager)
-        with self.assertRaisesRegex(TypeError, 'Argument type int is not iterable'):
-            list(counter(1))
-        with self.assertRaisesRegex(TypeError, 'Argument type bool is not iterable'):
-            list(counter([1, 2, 3], True))
-
-        # Expected
-        counter = MockBaseCounter(manager=self.manager)
-        rtn = counter([1, 2, 3])
-        self.assertIsInstance(rtn, GeneratorType)
-        self.assertEqual(list(rtn), [1, 2, 3])
-        self.assertEqual(counter.count, 3)
-
-        # Multiple arguments
-        counter = MockBaseCounter(manager=self.manager)
-        rtn = counter([1, 2, 3], (3, 2, 1))
-        self.assertIsInstance(rtn, GeneratorType)
-        self.assertEqual(tuple(rtn), (1, 2, 3, 3, 2, 1))
-        self.assertEqual(counter.count, 6)
-
-
 class TestSubCounter(TestCase):
     """
     Test the BaseCounter class
@@ -370,9 +234,11 @@ class TestCounter(TestCase):
         self.assertEqual(int(ctr.elapsed), 3)
 
     def test_refresh(self):
+        self.ctr.last_update = 0
         self.ctr.refresh()
         self.assertRegex(self.manager.output[0],
                          r'write\(output=%s, flush=True, position=3\)' % self.output)
+        self.assertAlmostEqual(self.ctr.last_update, time.time(), delta=0.3)
 
         self.manager.output = []
         self.ctr.refresh(flush=False)
@@ -385,8 +251,10 @@ class TestCounter(TestCase):
         self.assertEqual(len(self.manager.output), 0)
 
     def test_clear(self):
+        self.ctr.last_update = 100
         self.ctr.clear()
         self.assertRegex(self.manager.output[0], r'write\(output=, flush=True, position=3\)')
+        self.assertEqual(self.ctr.last_update, 0)
 
         self.manager.output = []
         self.ctr.clear(flush=False)
@@ -719,25 +587,25 @@ class TestCounter(TestCase):
         self.assertEqual(formatted, u'350 | 50 1.0 | 100')
 
     def test_close(self):
-        manager = mock.Mock()
+        manager = MockManager()
 
         # Clear is False
         ctr = MockCounter(manager=manager)
         ctr.close()
         self.assertEqual(ctr.calls, ['refresh(flush=True, elapsed=None)'])
-        self.assertEqual(manager.remove.call_count, 1)
+        self.assertEqual(manager.remove_calls, 1)
 
         # Clear is True, leave is True
         ctr = MockCounter(manager=manager, leave=True)
         ctr.close(clear=True)
         self.assertEqual(ctr.calls, ['refresh(flush=True, elapsed=None)'])
-        self.assertEqual(manager.remove.call_count, 2)
+        self.assertEqual(manager.remove_calls, 2)
 
         # Clear is True, leave is False
         ctr = MockCounter(manager=manager, leave=False)
         ctr.close(clear=True)
         self.assertEqual(ctr.calls, ['clear(flush=True)'])
-        self.assertEqual(manager.remove.call_count, 3)
+        self.assertEqual(manager.remove_calls, 3)
 
     def test_context_manager(self):
         mgr = Manager(stream=self.tty.stdout, enabled=False)
@@ -776,11 +644,11 @@ class TestCounter(TestCase):
         bar_format = ctr_format = u'{arg1:s} {count:d}'
 
         ctr = Counter(stream=self.tty.stdout, total=10, count=1, bar_format=bar_format,
-                      additional_fields={'arg1': 'hello'})
+                      fields={'arg1': 'hello'})
         self.assertEqual(ctr.format(), 'hello 1')
 
         ctr = Counter(stream=self.tty.stdout, count=1, counter_format=ctr_format,
-                      additional_fields={'arg1': 'hello'})
+                      fields={'arg1': 'hello'})
         self.assertEqual(ctr.format(), 'hello 1')
 
     def test_additional_fields_missing(self):
@@ -791,11 +659,11 @@ class TestCounter(TestCase):
         bar_format = ctr_format = u'{arg1:s} {count:d}'
 
         ctr = Counter(stream=self.tty.stdout, total=10, count=1, bar_format=bar_format)
-        with self.assertRaisesRegex(ValueError, "'arg1' specified in format, but not present"):
+        with self.assertRaisesRegex(ValueError, "'arg1' specified in format, but not provided"):
             ctr.format()
 
         ctr = Counter(stream=self.tty.stdout, count=1, counter_format=ctr_format)
-        with self.assertRaisesRegex(ValueError, "'arg1' specified in format, but not present"):
+        with self.assertRaisesRegex(ValueError, "'arg1' specified in format, but not provided"):
             ctr.format()
 
     def test_additional_fields_changed(self):
@@ -807,13 +675,13 @@ class TestCounter(TestCase):
         additional_fields = {'arg1': 'hello'}
 
         ctr = Counter(stream=self.tty.stdout, total=10, count=1, bar_format=bar_format,
-                      additional_fields=additional_fields)
+                      fields=additional_fields)
         self.assertEqual(ctr.format(), 'hello 1')
         additional_fields['arg1'] = 'goodbye'
         self.assertEqual(ctr.format(), 'goodbye 1')
 
         ctr = Counter(stream=self.tty.stdout, count=1, counter_format=ctr_format,
-                      additional_fields=additional_fields)
+                      fields=additional_fields)
         self.assertEqual(ctr.format(), 'goodbye 1')
         additional_fields['arg1'] = 'hello'
         self.assertEqual(ctr.format(), 'hello 1')
@@ -827,9 +695,70 @@ class TestCounter(TestCase):
         additional_fields = {'arg1': 'hello', 'count': 100000}
 
         ctr = Counter(stream=self.tty.stdout, total=10, count=1, bar_format=bar_format,
-                      additional_fields=additional_fields)
+                      fields=additional_fields)
         self.assertEqual(ctr.format(), 'hello 1')
 
         ctr = Counter(stream=self.tty.stdout, count=1, counter_format=ctr_format,
-                      additional_fields=additional_fields)
+                      fields=additional_fields)
         self.assertEqual(ctr.format(), 'hello 1')
+
+    def test_kwarg_fields(self):
+        """
+        Additional fields to format via keyword arguments
+        """
+
+        bar_format = ctr_format = u'{arg1:s} {count:d}'
+
+        ctr = Counter(stream=self.tty.stdout, total=10, count=1, bar_format=bar_format,
+                      arg1='hello')
+        self.assertEqual(ctr.format(), 'hello 1')
+
+        ctr.update(arg1='goodbye')
+        self.assertEqual(ctr.format(), 'goodbye 2')
+
+        ctr = Counter(stream=self.tty.stdout, count=1, counter_format=ctr_format,
+                      arg1='hello')
+        self.assertEqual(ctr.format(), 'hello 1')
+
+        ctr.update(arg1='goodbye')
+        self.assertEqual(ctr.format(), 'goodbye 2')
+
+    def test_kwarg_fields_precedence(self):
+        """
+        Keyword arguments take precedence over fields
+        """
+
+        bar_format = u'{arg1:s} {count:d}'
+        additional_fields = {'arg1': 'hello'}
+
+        ctr = Counter(stream=self.tty.stdout, total=10, count=1, bar_format=bar_format,
+                      fields=additional_fields)
+
+        self.assertEqual(ctr.format(), 'hello 1')
+
+        ctr.update(arg1='goodbye')
+        self.assertEqual(ctr.format(), 'goodbye 2')
+
+    def test_fill_setter(self):
+        """Fill must be one printable character"""
+
+        ctr = Counter(stream=self.tty.stdout, fill='a')
+
+        with self.assertRaisesRegex(ValueError, 'fill character must be a length of 1'):
+            ctr.fill = 'hello'
+
+        with self.assertRaisesRegex(ValueError, 'fill character must be a length of 1'):
+            ctr.fill = ''
+
+    def test_fill(self):
+        """
+        Fill uses remaining space
+        """
+
+        ctr_format = u'{fill}HI'
+        ctr = Counter(stream=self.tty.stdout, count=1, counter_format=ctr_format, fill=u'-')
+        self.assertEqual(ctr.format(), u'-' * 78 + 'HI')
+
+        ctr_format = u'{fill}HI{fill}'
+        ctr = Counter(stream=self.tty.stdout, count=1, counter_format=ctr_format, fill=u'-')
+        self.assertEqual(ctr.format(), u'-' * 39 + 'HI' + u'-' * 39)
