@@ -11,6 +11,7 @@ Test module for enlighten._manager
 
 import signal
 import sys
+import time
 
 from enlighten import _manager
 
@@ -192,6 +193,35 @@ class TestManager(TestCase):
                                     'Counter position 200 is greater than terminal height'):
             manager.counter(position=200)
 
+    def test_counter_position_pinned(self):
+        """If a position is taken, use next available"""
+
+        manager = _manager.Manager(stream=self.tty.stdout, set_scroll=False)
+        counter1 = manager.counter(position=2)
+        self.assertEqual(manager.counters[counter1], 2)
+
+        counter2 = manager.counter()
+        self.assertEqual(manager.counters[counter1], 2)
+        self.assertEqual(manager.counters[counter2], 1)
+
+        counter3 = manager.counter()
+        self.assertEqual(manager.counters[counter1], 2)
+        self.assertEqual(manager.counters[counter2], 3)
+        self.assertEqual(manager.counters[counter3], 1)
+
+        status1 = manager.status_bar(position=3)
+        self.assertEqual(manager.counters[counter1], 2)
+        self.assertEqual(manager.counters[counter2], 4)
+        self.assertEqual(manager.counters[counter3], 1)
+        self.assertEqual(manager.counters[status1], 3)
+
+        status2 = manager.status_bar()
+        self.assertEqual(manager.counters[counter1], 2)
+        self.assertEqual(manager.counters[counter2], 5)
+        self.assertEqual(manager.counters[counter3], 4)
+        self.assertEqual(manager.counters[status1], 3)
+        self.assertEqual(manager.counters[status2], 1)
+
     def test_inherit_kwargs(self):
         manager = _manager.Manager(stream=self.tty.stdout, counter_class=MockCounter,
                                    unit='knights', not_real=True, desc='Default')
@@ -212,14 +242,15 @@ class TestManager(TestCase):
 
         with mock.patch('enlighten._manager.Manager._set_scroll_area') as ssa:
             manager = _manager.Manager(stream=self.tty.stdout)
+            counter = manager.counter(position=3)
             term = manager.term
-            manager.write(msg, position=3)
+            manager.write(msg, counter=counter)
 
         self.tty.stdout.write(u'X\n')
         # Carriage return is getting converted to newline
         self.assertEqual(self.tty.stdread.readline(),
                          term.move(22, 0) + '\r' + term.clear_eol + msg + 'X\n')
-        self.assertEqual(ssa.call_count, 1)
+        self.assertEqual(ssa.call_count, 2)
 
     def test_write_no_flush(self):
         """
@@ -233,14 +264,43 @@ class TestManager(TestCase):
 
         with mock.patch('enlighten._manager.Manager._set_scroll_area') as ssa:
             manager = _manager.Manager(stream=self.tty.stdout)
+            counter = manager.counter(position=3)
             term = manager.term
-            manager.write(msg, position=3, flush=False)
+            manager.write(msg, counter=counter, flush=False)
 
         self.tty.stdout.write(u'X\n')
         # Carriage return is getting converted to newline
         self.assertEqual(self.tty.stdread.readline(),
                          term.move(22, 0) + '\r' + term.clear_eol + msg + 'X\n')
-        self.assertEqual(ssa.call_count, 1)
+        self.assertEqual(ssa.call_count, 2)
+
+    def test_autorefresh(self):
+        """
+        Ensure auto-refreshed counters are updated when others are
+        """
+
+        manager = _manager.Manager(stream=self.tty.stdout)
+        counter1 = manager.counter(count=1, total=0, counter_format=u'counter1', autorefresh=True)
+        counter2 = manager.counter(count=1, total=0, counter_format=u'counter2')
+        self.tty.clear()
+
+        # Counter 1 in auto-refresh list
+        self.assertIn(counter1, manager.autorefresh)
+
+        # If auto-refreshed counter hasn't been refreshed recently refresh
+        counter1.last_update = 0
+        counter2.refresh()
+        self.tty.stdout.write(u'X\n')
+        output = self.tty.stdread.readline()
+        self.assertRegex(output, 'counter2.+counter1')
+
+        # If auto-refreshed counter has been refreshed recently, skip
+        counter1.last_update = time.time() + 5
+        counter2.refresh()
+        self.tty.stdout.write(u'X\n')
+        output = self.tty.stdread.readline()
+        self.assertRegex(output, 'counter2')
+        self.assertNotRegex(output, 'counter1')
 
     def test_set_scroll_area_disabled(self):
         manager = _manager.Manager(stream=self.tty.stdout,
